/*****************************************************************************/
/*
                                  folder.c

Provide HTML form selection lists and buttons for VMS Mail files/folders.


COPYRIGHT
---------
Copyright (C) 2005-2008 Mark G.Daniel
This program, comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under the
conditions of the GNU GENERAL PUBLIC LICENSE, version 3, or any later version.


VERSION HISTORY
---------------
07-JUL-2008  MGD  refine PMDF IMAP folder (.MAILBOX) processing
27-JAN-2007  MGD  specially process PMDF IMAP folder subdirectories
30-DEC-2006  MGD  remove "(none)" folder list options
27-DEC-2006  MGD  FolderButtonListEach() simplify button width calculation
21-OCT-2006  MGD  multiple Mail file [copy][move] destination selectors
                    provide all files (drawers) and associated folders 
20-JUL-2006  MGD  try to make font width allowances for monospace
04-JUL-2006  MGD  FolderButtonListEach() refine (perhaps) button width kludge
09-APR-2006  MGD  FolderOpenSingleFileListEach() support public wildcard
                    folder configuration with MAIL and NEWMAIL prohibition
06-APR-2006  MGD  remove default MAIL and NEWMAIL folders
02-APR-2006  MGD  FolderOpenSelect() modifications for public wildcard access
07-FEB-2006  MGD  folder buttons for left-side of message listing
01-FEB-2005  MGD  initial
*/

/*****************************************************************************/

#ifdef SOYMAIL_VMS_V6
#undef _VMS_V6_SOURCE
#define _VMS_V6_SOURCE
#undef __VMS_VER
#define __VMS_VER 60000000
#undef __CRTL_VER
#define __CRTL_VER 60000000
#endif

#ifndef __VAX
#   pragma nomember_alignment
#endif

/* standard C header files */
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* VMS related header files */
#include <descrip.h>
#include <mailmsgdef.h>
#include <ssdef.h>
#include <stsdef.h>

/* application header file */
#include "soymail.h"
#include "attach.h"
#include "callmail.h"
#include "cgilib.h"
#include "compose.h"
#include "folder.h"
#include "mainmenu.h"
#include "newmail.h"
#include "other.h"
#include "message.h"

#define FI_LI __FILE__, __LINE__

/********************/
/* external storage */
/********************/

extern BOOL  Debug,
             WatchEnabled;

extern int  VmsVersion;

extern char  CurrentVmsTimeString[],
             SoftwareId[],
             SoftwareVn[];

extern USER_OPTIONS  UserOptions;
extern REQUEST_DATA  RequestData;
extern VMS_MAIL_USER  VmsMailUser;


/*****************************************************************************/
/*
Reset (empty) the mail file and folder lists.
Called upon certain triggers and periodically.
*/

void FolderListReset (REQUEST_DATA *rdptr)

{
   int  cnt, status;
   char  *cptr, *sptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   muptr = &VmsMailUser;

   if (WatchEnabled) WatchThis ("FolderListReset()");

   if (muptr->MailFolderListPtr)
   {
      free (muptr->MailFolderListPtr);
      muptr->MailFolderListPtr = NULL;
      muptr->MailFolderListSize =
         muptr->MailFolderListLength =
         muptr->MailFolderListCount = 0;
   }

   if (muptr->DestFolderListPtr)
   {
      free (muptr->DestFolderListPtr);
      muptr->DestFolderListPtr = NULL;
      muptr->DestFolderListSize =
         muptr->DestFolderListLength =
         muptr->DestFolderListCount = 0;
   }
}

/*****************************************************************************/
/*
Get the list of mail files and a list of folders in the default and destination
mail file (as applicable).  This is quite messy as only one mail file may be
open at a time so we must dance around the primary mail file and folder list.
*/

void FolderListInit (REQUEST_DATA *rdptr)

{
   int  cnt, len, status;
   char  *cptr;
   USER_OPTIONS  *uoptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   muptr = &VmsMailUser;
   uoptr = &rdptr->UserOptions;

   if (WatchEnabled)
      WatchThis ("FolderListInit() all:!UL pmdf:!UL",
                 uoptr->AllMailFiles, uoptr->PmdfImap);

   if (uoptr->AllMailFiles && !muptr->MailFileListPtr)
      CallMailFileList (muptr);

   if (!muptr->MailFolderListPtr)
   {
      status = CallMailFolderList (muptr, FALSE, uoptr->PmdfImap);
      if (VMSnok (status))
      {
         if (muptr->MailVmsStatus == MAIL$_OPENIN &&
             muptr->MailFileDeletedCount)
            StatusMessage (FI_LI, -1, LangFor("mail_file_deleted"),
                           rdptr->MailFileName);
         else
         if (uoptr->PmdfImap && (status & 0x80000000))
            StatusMessage (FI_LI, 1, "IMAP: %s.",
                           SysGetMsg(status & 0x7fffffff));
         else
            StatusMessage (FI_LI, 1, "MAIL: %s.", SysGetMsg(status));
      }
      time ((unsigned long*)&rdptr->FolderRefreshSeconds);
   }

   if (!muptr->DestFolderListPtr)
   {
      if (!uoptr->AllMailFiles ||
          !strcmp (rdptr->MailFileName, rdptr->DestMailFileName))
      {
         /* clone the mail folder list */
         if (WatchEnabled) WatchThis ("clone");
         muptr->DestFolderListCount = muptr->MailFolderListCount;
         muptr->DestFolderListLength = muptr->MailFolderListLength;
         muptr->DestFolderListSize = muptr->MailFolderListLength + 1;
         muptr->DestFolderListPtr = calloc (1, muptr->DestFolderListSize);
         if (!muptr->DestFolderListPtr) ErrorExit (vaxc$errno, FI_LI);
         if (muptr->MailFolderListPtr)
            memcpy (muptr->DestFolderListPtr,
                    muptr->MailFolderListPtr,
                    muptr->DestFolderListSize);
         time ((unsigned long*)&rdptr->FolderRefreshSeconds);
      }
      else
      {
         /* will handle all the opening and closing of the dest mail file */
         status = CallMailFolderList (muptr, TRUE, uoptr->PmdfImap);
         if (VMSnok (status))
            /* quite possible a specified mail file does not exist (yet) */
            if (status != MAIL$_OPENIN)
               StatusMessage (FI_LI, 1, "MAIL: %s.", SysGetMsg(status));

         time ((unsigned long*)&rdptr->FolderRefreshSeconds);
      }
   }

   /* find the length of the longest mail file name */
   muptr->MailFolderListMaxLen = 0;
   cptr = muptr->MailFileListPtr;
   for (cnt = muptr->MailFileListCount; cnt; cnt--)
   {
      if ((len = strlen(cptr)) > muptr->MailFolderListMaxLen)
         muptr->MailFolderListMaxLen = len;
      while (*cptr) cptr++;
      cptr++;
   }
   muptr->DestFolderListMaxLen = muptr->MailFolderListMaxLen;

   /* integrate with the length of the longest folder name */
   cptr = muptr->MailFolderListPtr;
   for (cnt = muptr->MailFolderListCount; cnt; cnt--)
   {
      if ((len = strlen(cptr)) > muptr->MailFolderListMaxLen)
         muptr->MailFolderListMaxLen = len;
      while (*cptr) cptr++;
      cptr++;
   }

   /* and with the length of the longest destination folder name */
   cptr = muptr->DestFolderListPtr;
   for (cnt = muptr->DestFolderListCount; cnt; cnt--)
   {
      if ((len = strlen(cptr)) > muptr->DestFolderListMaxLen)
         muptr->DestFolderListMaxLen = len;
      while (*cptr) cptr++;
      cptr++;
   }

   if (WatchEnabled)
   {
      fprintf (stdout, "MAIL: %d (%d)\n",
               muptr->MailFolderListCount, muptr->MailFolderListMaxLen);
      cptr = muptr->MailFolderListPtr;
      for (cnt = muptr->MailFolderListCount; cnt; cnt--)
      {
         fprintf (stdout, "%2d |%s|\n", strlen(cptr), cptr);
         while (*cptr) cptr++;
         cptr++;
      }
      fprintf (stdout, "DEST: %d (%d)\n",
               muptr->DestFolderListCount, muptr->DestFolderListMaxLen);
      cptr = muptr->DestFolderListPtr;
      for (cnt = muptr->DestFolderListCount; cnt; cnt--)
      {
         fprintf (stdout, "%2d |%s|\n", strlen(cptr), cptr);
         while (*cptr) cptr++;
         cptr++;
      }
   }
}

/*****************************************************************************/
/*
Generate a selection list for the available mail folders and [open] button. 
If private access then include the [9 new] button.
*/

void FolderOpenSelect (REQUEST_DATA *rdptr)

{
   int  status;
   char  newBuf [256];
   USER_OPTIONS  *uoptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderOpenSelect()");

   muptr = &VmsMailUser;
   uoptr = &rdptr->UserOptions;

   fprintf (stdout,
"<!-- folder open begin -->\n\
<INPUT TYPE=\"hidden\" NAME=\"folder_list_change\" \
ID=\"folder_list_change\" VALUE=\"\">\n\
<SELECT CLASS=\"css_select_2\" NAME=\"folder_list_open\"%s \
ONCHANGE=\"document.getElementById(\'folder_list_change\').value=true;\
startWorking();document.getElementById(\'soymail_form\').submit();\">\n",
            muptr->MailFolderListMaxLen > 20 ? " STYLE=\"width:12em;\"" : "");

   FolderOpenListEach (rdptr);

   sprintf (newBuf, "%d %s", rdptr->NewMessages, LangFor("new"));

   fprintf (stdout,
"</SELECT>&nbsp;&nbsp;\n\
<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\"%s>&nbsp;\n",
            LangFor("open"),
            CLICK_STARTWORKING);

   if (rdptr->PrivateAccess)
      fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\"%s>&nbsp; \n",
               newBuf,
               NewMailInclude(rdptr) ?
                  " ONCLICK=\"refreshNewMail(-1);return false;\"" :
                  CLICK_STARTWORKING);

   fprintf (stdout,
"<!-- folder open end -->\n\n");
}

/*****************************************************************************/
/*
Provide a selection list for the message folders.
*/

void FolderOpenListEach (REQUEST_DATA *rdptr)

{
   int  cnt;
   char  *cptr, *sptr, *tptr;
   USER_OPTIONS  *uoptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderOpenListEach()");

   muptr = &VmsMailUser;
   uoptr = &rdptr->UserOptions;

   if (uoptr->PmdfImap)
      FolderPmdfImapListEach (rdptr, FALSE);
   else
   if (uoptr->AllMailFiles || muptr->MailFileListCount > 1)
      FolderMultiFileListEach (rdptr, FALSE);
   else
      FolderSingleFileListEach (rdptr, FALSE);
}

/*****************************************************************************/
/*
Provide a selection list for the message action destination folders.
*/

void FolderDestListEach (REQUEST_DATA *rdptr)

{
   int  cnt;
   char  *cptr, *sptr, *tptr;
   USER_OPTIONS  *uoptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderDestListEach()");

   muptr = &VmsMailUser;
   uoptr = &rdptr->UserOptions;

   if (uoptr->PmdfImap)
      FolderPmdfImapListEach (rdptr, TRUE);
   else
   if (uoptr->AllMailFiles || muptr->MailFileListCount > 1)
      FolderMultiFileListEach (rdptr, TRUE);
   else
      FolderSingleFileListEach (rdptr, TRUE);
}

/*****************************************************************************/
/*
Only a single MAIL.MAI file or optioned to only show MAIL.MAI.
List each folder.
*/

void FolderSingleFileListEach
(
REQUEST_DATA *rdptr,
BOOL DestFolder
)
{
   int  fldcnt,
        ListCount;
   char  *cptr, *fldptr, *selptr,
         *FolderNamePtr,
         *ListPtr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderSingleFileListEach()");

   muptr = &VmsMailUser;

   if (DestFolder)
   {
       ListPtr = muptr->DestFolderListPtr;
       ListCount = muptr->DestFolderListCount;
       FolderNamePtr = rdptr->DestFolderName;
   }
   else
   {
       ListPtr = muptr->MailFolderListPtr;
       ListCount = muptr->MailFolderListCount;
       FolderNamePtr = rdptr->FolderName;
   }

   if (!rdptr->NewMessages && !strcmp (FolderNamePtr, "NEWMAIL"))
      fputs ("<OPTION VALUE=\"NEWMAIL\" STYLE=\"padding-left:0.5em;\" \
SELECTED>NEWMAIL\n", stdout);

   fldptr = ListPtr;
   for (fldcnt = ListCount; fldcnt; fldcnt--)
   {
      if (rdptr->PublicWildcard &&
          rdptr->PublicNoNewMail &&
          (!strcmp (fldptr, "MAIL") ||
           !strcmp (fldptr, "NEWMAIL")))
      {
         /* next folder in list */
         while (*fldptr) fldptr++;
         fldptr++;
         continue;
      }

      if (!strcmp (fldptr, FolderNamePtr))
         selptr = " SELECTED";
      else
         selptr = "";

      fprintf (stdout,
"<OPTION VALUE=\"%s\" STYLE=\"padding-left:0.5em;\"%s>%s\n",
               HTML_ESCAPE(fldptr), selptr, HTML_ESCAPE(fldptr));

      /* next folder in list */
      while (*fldptr) fldptr++;
      fldptr++;
   }
}

/*****************************************************************************/
/*
Multiple mail .MAI files.
List each folder for the currently selected mail .MAI file.
*/

void FolderMultiFileListEach
(
REQUEST_DATA *rdptr,
BOOL DestFolder
)
{
   int  filcnt, fldcnt,
        ListCount;
   char  *cptr, *filptr, *fldptr, *selptr,
         *FolderNamePtr,
         *ListPtr,
         *MailFilePtr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderMultiFileListEach()");

   muptr = &VmsMailUser;

   if (DestFolder)
   {
       FolderNamePtr = rdptr->DestFolderName;
       ListCount = muptr->DestFolderListCount;
       ListPtr = muptr->DestFolderListPtr;
       MailFilePtr = rdptr->DestMailFileName;
   }
   else
   {
       FolderNamePtr = rdptr->FolderName;
       ListCount = muptr->MailFolderListCount;
       ListPtr = muptr->MailFolderListPtr;
       MailFilePtr = rdptr->MailFileName;
   }

   filptr = muptr->MailFileListPtr;
   for (filcnt = muptr->MailFileListCount; filcnt; filcnt--)
   {
      if (!strcmp (filptr, MailFilePtr))
      {
         /* current mail file */
         fprintf (stdout, "<OPTION VALUE=\"%s/MAIL\" SELECTED>%s/\n",
                  filptr, filptr);
         fldptr = ListPtr;
         for (fldcnt = ListCount; fldcnt; fldcnt--)
         {
            if (!strcmp (fldptr, FolderNamePtr))
               selptr = " SELECTED";
            else
               selptr = "";

            /* Opera 9 doesn't notice the style padding hence the &nbsp; */
            fprintf (stdout,
"<OPTION VALUE=\"%s/%s\" STYLE=\"padding-left:1em;\"%s>&nbsp;&nbsp;/%s\n",
                     filptr, HTML_ESCAPE(fldptr), selptr, HTML_ESCAPE(fldptr));

            /* next folder name in the list */
            while (*fldptr) fldptr++;
            fldptr++;
         }
      }
      else
      {
         /* show an empty folder for each mail file */
         fprintf (stdout, "<OPTION VALUE=\"%s/MAIL\">%s/\n",
                  filptr, filptr);
      }

      /* next mail file name in the list */
      while (*filptr) filptr++;
      filptr++;
   }
}

/*****************************************************************************/
/*
List each folder in the PMDF IMAP mailbox.
*/

void FolderPmdfImapListEach
(
REQUEST_DATA *rdptr,
BOOL DestFolder
)
{
   int  fldcnt,
        ListCount;
   char  *cptr, *fldptr, *selptr, *sptr, *zptr,
         *FileNamePtr,
         *FolderNamePtr,
         *ListPtr;
   char  FolderName [256];
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderPmdfImapListEach()");

   muptr = &VmsMailUser;

   if (DestFolder)
   {
       ListPtr = muptr->DestFolderListPtr;
       ListCount = muptr->DestFolderListCount;
       FileNamePtr = rdptr->DestMailFileName;
       FolderNamePtr = rdptr->DestFolderName;
   }
   else
   {
       ListPtr = muptr->MailFolderListPtr;
       ListCount = muptr->MailFolderListCount;
       FileNamePtr = rdptr->MailFileName;
       FolderNamePtr = rdptr->FolderName;
   }

   /* check for the PMDF IMAP indicative trailing slash */
   for (cptr = FileNamePtr; *cptr && *cptr != '/'; cptr++);
   if (!*cptr) FileNamePtr = NULL;

   fldptr = ListPtr;
   for (fldcnt = ListCount; fldcnt; fldcnt--)
   {
      if (FileNamePtr)
      {
         /* PMDF IMAP 'mail file' so always compare using that */
         zptr = (sptr = FolderName) + sizeof(FolderName)-1;
         for (cptr = FileNamePtr; *cptr && sptr < zptr; *sptr++ = *cptr++);
         for (cptr = FolderNamePtr; *cptr && sptr < zptr; *sptr++ = *cptr++);
         *sptr = '\0';
         if (!strcmp (fldptr, FolderName))
            selptr = " SELECTED";
         else
            selptr = "";
      }
      else
      {
         if (!strcmp (fldptr, FolderNamePtr))
            selptr = " SELECTED";
         else
            selptr = "";
      }

      for (cptr = fldptr; *cptr && *cptr != '/'; cptr++);

      fprintf (stdout,
"<OPTION VALUE=\"%s%s\" STYLE=\"padding-left:0.5em;\"%s>%s\n",
                *cptr ? "" : "MAIL/",
                HTML_ESCAPE(fldptr), selptr,
                HTML_ESCAPE(fldptr));

      /* next folder in list */
      while (*fldptr) fldptr++;
      fldptr++;
   }
}

/*****************************************************************************/
/*
Differences in the CSS rendering of (only) buttons inside table data (the CSS
lay-up technicalities are well outside any expertise I might have) for Firefox
1.5, Mozilla 1.7, Opera 8.5 and MSIE 6.0 make this guesstimate of the width of
the required buttons absolutely necessary.  I played around with the various
bits of CSS I understood and still got vastly differing behaviours between the
agents and eventually resorted to this nightmare.  Talk about making it up as
you go along!
*/

void FolderButtonListEach (REQUEST_DATA *rdptr)

{
   int  ButtonWidth;
   float  WidthFactor;
   USER_OPTIONS  *uoptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderButtonListEach()");

   muptr = &VmsMailUser;
   uoptr = &rdptr->UserOptions;

   ButtonWidth = muptr->MailFolderListMaxLen;

   if (ButtonWidth > 30)
      WidthFactor = 0.70;
   else
   if (ButtonWidth > 20)
      WidthFactor = 0.65;
   else
   if (ButtonWidth > 10)
      WidthFactor = 0.80;
   else
      WidthFactor = 1.10;

   /* MSIE6 seems to render fonts slightly larger than the others tested */
   if (rdptr->UserAgent == SOY_USER_AGENT_MSIE) WidthFactor += 0.1;

   /* who uses monospace?! (at least the one person who reported issues :-) */
   if (uoptr->FontIsMono) WidthFactor += 0.15;

   ButtonWidth = (int)((float)ButtonWidth * WidthFactor);

   if (uoptr->PmdfImap)
      FolderButtonPmdfImapListEach (rdptr, ButtonWidth);
   else
   if (uoptr->AllMailFiles || muptr->MailFileListCount > 1)
      FolderButtonMultiFileListEach (rdptr, ButtonWidth);
   else
      FolderButtonSingleFileListEach (rdptr, ButtonWidth);
}

/*****************************************************************************/
/*
Only a single MAIL.MAI file or optioned to only show MAIL.MAI.
Provide a button for each folder.
*/

void FolderButtonSingleFileListEach
(
REQUEST_DATA *rdptr,
int ButtonWidth
)
{
   int  cnt, fldcnt, idx;
   char  *cptr, *fldptr;
   USER_OPTIONS  *uoptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderButtonSingleFileListEach()");

   muptr = &VmsMailUser;
   uoptr = &rdptr->UserOptions;

   cnt = 0;
   fldptr = muptr->MailFolderListPtr;
   for (fldcnt = muptr->MailFolderListCount; fldcnt; fldcnt--)
   {
      if (cnt && !(cnt % uoptr->FolderButtons)) 
         fprintf (stdout, "<P CLASS=\"css_p02\">");
      cnt++;

      fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_1\" \
STYLE=\"width:%dem;text-align:left;padding-left:0.5em;\" \
TYPE=\"submit\" NAME=\"folder_btn\" VALUE=\"%s\">\n",
               ButtonWidth, HTML_ESCAPE(fldptr));

      /* next folder in list */
      while (*fldptr) fldptr++;
      fldptr++;
   }
}

/*****************************************************************************/
/*
Multiple mail .MAI files.
Provide a button for each mail file and each folder for the currently selected
mail .MAI file.
*/

void FolderButtonMultiFileListEach
(
REQUEST_DATA *rdptr,
int ButtonWidth
)
{
   int  cnt, filcnt, fldcnt, idx;
   char  *cptr, *filptr, *fldptr;
   USER_OPTIONS  *uoptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderButtonMultiFileListEach()");

   muptr = &VmsMailUser;
   uoptr = &rdptr->UserOptions;

   /* folder buttons don't contain the mail file, supply using this */
   fprintf (stdout,
"<INPUT TYPE=\"hidden\" NAME=\"folder_btn_mailfile\" VALUE=\"%s\">\n",
            rdptr->MailFileName); 

   cnt = 0;

   filptr = muptr->MailFileListPtr;
   for (filcnt = muptr->MailFileListCount; filcnt; filcnt--)
   {
      if (cnt && !(cnt % uoptr->FolderButtons)) 
         fprintf (stdout, "<P CLASS=\"css_p02\">");
      cnt++;

      /* show a button for each mail file */
      if (rdptr->ClientHasJavaScript)
         fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_1\" \
STYLE=\"width:%dem;text-align:left;padding-left:0.5em;\" \
TYPE=\"submit\" NAME=\"folder_btn\" VALUE=\"%s/\" \
ONCLICK=\"this.value=\'%s/\';return true;\">\n",
               ButtonWidth, filptr, filptr);
      else
         fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_1\" \
STYLE=\"width:%dem;text-align:left;padding-left:0.5em;\" \
TYPE=\"submit\" NAME=\"folder_btn\" VALUE=\"%s/\">\n",
               ButtonWidth, filptr);

      if (!strcmp (filptr, rdptr->MailFileName))
      {
         /* current mail file */
         fldptr = muptr->MailFolderListPtr;
         for (fldcnt = muptr->MailFolderListCount; fldcnt; fldcnt--)
         {
            if (cnt && !(cnt % uoptr->FolderButtons)) 
               fprintf (stdout, "<P CLASS=\"css_p02\">");
            cnt++;

            fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_1\" \
STYLE=\"width:%dem;text-align:left;padding-left:2em;\" \
TYPE=\"submit\" NAME=\"folder_btn\" VALUE=\"/%s\">\n",
                     ButtonWidth, HTML_ESCAPE(fldptr));

            /* next folder name in the list */
            while (*fldptr) fldptr++;
            fldptr++;
         }
      }

      /* next mail file name in the list */
      while (*filptr) filptr++;
      filptr++;
   }
}

/*****************************************************************************/
/*
List each folder in the PMDF IMAP mailbox.
*/

void FolderButtonPmdfImapListEach
(
REQUEST_DATA *rdptr,
int ButtonWidth
)
{
   int  cnt, fldcnt,
        SlashCount;
   char  *cptr, *sptr, *zptr, *fldptr;
   char  MailFileName [128];
   USER_OPTIONS  *uoptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderButtonPmdfImapListEach()");

   muptr = &VmsMailUser;
   uoptr = &rdptr->UserOptions;

   cnt = 0;

   fldptr = muptr->MailFolderListPtr;
   for (fldcnt = muptr->MailFolderListCount; fldcnt; fldcnt--)
   {
      if (cnt && !(cnt % uoptr->FolderButtons)) 
         fprintf (stdout, "<P CLASS=\"css_p02\">");
      cnt++;

      zptr = (sptr = MailFileName) + sizeof(MailFileName)-1;
      SlashCount = 0;
      for (cptr = fldptr; *cptr; cptr++) if (*cptr == '/') SlashCount++;
      if (SlashCount)
      {
         /* hint: a PMDF IMAP mail file will contain at least one slash */
         while (*fldptr && SlashCount && sptr < zptr)
         {
            if (*fldptr == '/') SlashCount--;
            *sptr++ = *fldptr++;
         }
      }
      else
         for (cptr = "MAIL"; *cptr && sptr < zptr; *sptr++ = *cptr++);
      *sptr = '\0';

      if (!strcmp (MailFileName, "MAIL"))
         fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_1\" \
STYLE=\"width:%dem;text-align:left;padding-left:2em;\" \
TYPE=\"submit\" NAME=\"folder_btn\" VALUE=\"%s\">\n",
                  ButtonWidth, HTML_ESCAPE(fldptr));
      else
         fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_1\" \
STYLE=\"width:%dem;text-align:left;padding-left:2em;\" \
TYPE=\"submit\" NAME=\"folder_btn\" VALUE=\"%s/%s\">\n",
                  ButtonWidth, HTML_ESCAPE(MailFileName),
                  HTML_ESCAPE(fldptr));

      /* next folder in list */
      while (*fldptr) fldptr++;
      fldptr++;
   }
}

/*****************************************************************************/
/*
A faked folder select specifically for the help menu bar page.
*/

void FolderOpenHelp (REQUEST_DATA *rdptr)

{
   int  status;
   char  newBuf [256];
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderOpenHelp()");

   fprintf (stdout,
"<!-- folder open help begin -->\n\
<TD NOWRAP ALIGN=\"left\" VALIGN=\"middle\">\n\
<SELECT CLASS=\"css_select_2\" NAME=\"folder_list_help\">\n\
<OPTION VALUE=\"%s\" SELECTED>&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n\
</SELECT>&nbsp;&nbsp;\n\
<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\">&nbsp; \n\
<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"0 %s\">&nbsp; \n\
</TD>\n\
<!-- folder open help end -->\n\n",
            LangFor("help"), LangFor("help"),
            LangFor("open"),
            LangFor("new"));
}

/*****************************************************************************/
/*
Generate an appropriate folder description (i.e. if it's not the default mail
file then include it in the description) and return a pointer to dynamic
storage.
*/

char* FolderDescribe
(
char *MailFileName,
char *FolderName
)
{
   char  *cptr, *sptr, *zptr;
   char  buf [256];

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("FolderDescribe()");

   zptr = (sptr = buf) + sizeof(buf)-1;
   if (MailFileName)
   {
      if (strcmp (MailFileName, "MAIL"))
      {
         /* not the default MAIL.MAI */
         for (cptr = MailFileName; *cptr && sptr < zptr; *sptr++ = *cptr++);
         if (sptr > buf && *(sptr-1) == '/') sptr--;
         if (sptr < zptr) *sptr++ = '/';
      }
   }
   if (FolderName)
      for (cptr = FolderName; *cptr && sptr < zptr; *sptr++ = *cptr++);
   *sptr = '\0';
   cptr = calloc (1, sptr-buf+1);
   strcpy (cptr, buf);

   if (WatchEnabled) WatchThis ("!AZ", cptr);

   return (cptr);
}

/*****************************************************************************/

