/*****************************************************************************/
/*
                                 mainmenu.c

The soyMAIL main menu is the panel present across the top of most pages,
providing the folder selection [folder][Open], [New 999], [Search], [Options],
[Contacts], [Compose], [Logout], [close] and [help] buttons.  This module also
provides the status panel.


COPYRIGHT
---------
Copyright (C) 2005-2008 Mark G.Daniel
This program, comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under the
conditions of the GNU GENERAL PUBLIC LICENSE, version 3, or any later version.


VERSION HISTORY
---------------
07-OCT-2006  MGD  soyMAIL autogenous authentication and logout
12-JUN-2006  MGD  add form field CLIENT_TIME and JavaScript clientTime()
                    to provide local (to browser) message origination time
16-MAR-2006  MGD  revised [logout] support
27-FEB-2006  MGD  [close] now has a JavaScript confirm for compose page
25-FEB-2006  MGD  [logout] now has associated JavaScript confirm explaining 
                    the logout basics and allowing an opportunity to cancel
                  bugfix; MainMenuRequest() FOLDER_LIST_CHANGE detect 't'
14-FEB-2006  MGD  main menu [compose] button now has an associated [^] popup
                  main menu [close] button for child windows
01-FEB-2005  MGD  initial
*/

/*****************************************************************************/

#ifdef SOYMAIL_VMS_V6
#undef _VMS_V6_SOURCE
#define _VMS_V6_SOURCE
#undef __VMS_VER
#define __VMS_VER 60000000
#undef __CRTL_VER
#define __CRTL_VER 60000000
#endif

#ifndef __VAX
#   pragma nomember_alignment
#endif

/* standard C header files */
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* VMS related header files */
#include <descrip.h>
#include <mailmsgdef.h>
#include <ssdef.h>
#include <stsdef.h>

/* application header file */
#include "soymail.h"
#include "callmail.h"
#include "cereal.h"
#include "cgilib.h"
#include "config.h"
#include "login.h"
#include "mainmenu.h"
#include "newmail.h"
#include "other.h"
#include "message.h"

#define FI_LI __FILE__, __LINE__

/* global storage */

/* external storage */

extern BOOL  Debug,
             WatchEnabled;

extern int  VmsVersion;

extern char  *CgiEnvironmentPtr,
             *SoyMailStatusPtr;

extern char  SoftwareCopy[],
             SoftwareEnv[],
             SoftwareVn[];

extern CONFIG_DATA  SoyMailConfig;
extern USER_OPTIONS  UserOptions;
extern VMS_MAIL_USER  VmsMailUser;

/* ad hoc prototypes */

int lib$day_of_week(__unknown_params);
int sys$fao(__unknown_params);

/****************************************************************************/
/*
Parse the buttons from the main menu.
*/ 

BOOL MainMenuRequest (REQUEST_DATA *rdptr)

{
   /* must be static because it's pointed to for possible use later */
   static char  bbuf [64];

   char  *cptr, *sptr, *zptr;
   USER_OPTIONS  *uoptr;
   REQUEST_DATA  *sdptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuRequest()");

   /* a bit of sanity checking */
   if (!rdptr->PrivateAccess) ErrorExit (SS$_BUGCHECK, FI_LI);

   uoptr = &rdptr->UserOptions;
   sdptr = (REQUEST_DATA*)rdptr->StateDataPtr;

   CGIVARNULL (cptr, "FORM_FOLDER_LIST_CHANGE");
   if (cptr && *cptr == 't')
   {
      /* folder selection (change) will propagate a search, [open] doesn't */
      MessageFolderPage (rdptr);
      return (TRUE);
   }

   CGIVARNULL (cptr, "FORM_NEWMAIL_REFRESH");
   if (cptr && *cptr == 'N')
   {
      NewMailMessage (rdptr);
      return (TRUE);
   }

   CGIVARNULL (cptr, "FORM_MAIN_BTN");
   if (!cptr) return (FALSE);

   rdptr->SearchData.InProgress = FALSE;

   /* we insert new message count into "new" button! */
   for (sptr = cptr; *sptr && !isdigit(*sptr); sptr++);
   if (*sptr)
   {
      cptr = sptr;
      while (*cptr && isdigit(*cptr)) cptr++;
      while (*cptr && isspace(*cptr)) cptr++;
      zptr = (sptr = bbuf) + sizeof(bbuf)-1;
      while (*cptr && sptr < zptr) *sptr++ = *cptr++;
      if (sptr >= zptr) ErrorExit (SS$_BUGCHECK, FI_LI);
      sptr--;
      while (sptr > bbuf && isdigit(*sptr)) sptr--;
      *++sptr = '\0';
      cptr = bbuf;
   }

   if (LangSame ("open", cptr))
   {
      MessageFolderPage (rdptr);
      return (TRUE);
   }

   if (LangSame ("new", cptr))
   {
      NewMailMessage (rdptr);
      return (TRUE);
   }

   /* searching can be disabled */
   if (LangSame ("search", cptr) ||
       LangSame ("search_new", cptr))
      return (SearchPageRequest (rdptr));

   if (LangSame ("options", cptr))
   {
      OptionsPageRequest (rdptr);
      return (TRUE);
   }

   if (LangSame ("contacts", cptr))
   {
      ContactsPageRequest (rdptr);
      return (TRUE);
   }

   /* [compose] is the only button currently to have an associated [^] popup */
   if (LangSame ("compose", cptr) || *cptr == '^')
   {
      ComposePageRequest (rdptr);
      return (TRUE);
   }

   if (LangSame ("logout", cptr))
   {
      if (SoyMailConfig.LogoutDisabled)
      {
         StatusMessage (FI_LI, 1, LangFor("logout_disabled"));
         return (FALSE);
      }
      if (SoyMailConfig.LoginSecret)
      {
         /* using soyMAIL's autogenous authentication, force re-login */
         LoginPage (rdptr, "", NULL, 0);
         RequestAccessLog (NULL, "logout");
      }
      else
      {
         strcat (rdptr->FormAction, SOY_LOGOUT_SENTINAL);
         CgiLibResponseRedirect (rdptr->FormAction);
      }
      return (TRUE);
   }

   if (LangSame ("help", cptr))
   {
      HelpPageRequest (rdptr);
      return (TRUE);
   }

   return (FALSE);
}

/*****************************************************************************/
/*
*/

void MainMenuPageBegin
(
REQUEST_DATA *rdptr,
int ThisPageIdent
)
{
   int  NewMessages,
        SoyMailHistory;
   char  *cptr,
         *OperaChildPtr,
         *UserNamePtr;
   USER_OPTIONS  *uoptr;
   REQUEST_DATA  *sdptr;
   VMS_MAIL_MSG  *vmptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuPageBegin()");

   sdptr = (REQUEST_DATA*)rdptr->StateDataPtr;
   uoptr = &rdptr->UserOptions;
   muptr = &VmsMailUser;
   vmptr = &muptr->VmsMailMsg;

   if (!rdptr->ThisPageIdent) rdptr->ThisPageIdent = ThisPageIdent;

   if (rdptr->CharSetPtr) CgiLibResponseSetCharset (rdptr->CharSetPtr);

   CgiLibResponseHeader (200, "text/html", "%s", rdptr->LoginSetCookiePtr);

   fprintf (stdout,
"<HTML>\n\
<HEAD>\n\
<META NAME=\"generator\" CONTENT=\"%s\">\n\
<META NAME=\"environment\" CONTENT=\"%s\">\n\
<META NAME=\"copyright\" CONTENT=\"%s - GPL licensed\">\n\
<META NAME=\"language\" CONTENT=\"%s %s\">\n",
            SoftwareEnv,
            CgiEnvironmentPtr,
            SoftwareCopy,
            LangFor("lang_name"),
            LangFor("lang_author"));

   /* used to support some of the enhanced functionality */
   SoyMailJavaScriptStuff (rdptr);

   /* returns a pointer to the '<BODY ONLOAD=()>' requirement */
   cptr = NewMailScriptRefresh (rdptr);

   SoyMailStyleStuff (rdptr, uoptr->Theme);

   fprintf (stdout,
"<TITLE>%s</TITLE>\n\
</HEAD>\n\n",
            SoyMailPageTitle(rdptr));

   if (ThisPageIdent == PAGE_HELP)
      fprintf (stdout, "<BODY ONLOAD=\"refreshClock();window.focus();\">\n");
   else
   if (ThisPageIdent == PAGE_OPTIONS)
      fprintf (stdout, "<BODY ONLOAD=\"refreshClock();resizePanels();\">\n");
   else
   {
      fprintf (stdout, "<BODY ONLOAD=\"refreshClock();");

      /* use the pointer to the '<BODY onload=()>' requirement */
      if (cptr) fputs (cptr, stdout);

      if (rdptr->NewMailPlay)
      {
         fputs ("playYouGotMail();", stdout);
         rdptr->NewMailPlay = FALSE;
      }

      if (ThisPageIdent == PAGE_COMPOSE)
      {
         fprintf (stdout, "contactListResize();");
         if (rdptr->ComposeData.EditorType == COMPOSE_EDITOR_HTML)
            fprintf (stdout, "htmlEditorLoad();");
      }

      if (rdptr->FieldFocusPtr) fputs (rdptr->FieldFocusPtr, stdout);

      fprintf (stdout, "\"");

      fprintf (stdout,
" ONUNLOAD=\"clearWorking();\" \
ONFOCUS=\"refreshSoyMailForm();\" \
ONBLUR=\"refreshSoyMailForm();\"");

      fprintf (stdout, ">\n");
   }

   /* keeps track of previous soyMAIL usage history for (re-)login purposes */
   CGIVAR (cptr, "FORM_SOYMAIL_HISTORY");
   SoyMailHistory = atoi(cptr) + 1;

   /* see description in SoyMailJavaScriptStuff() */
   if (!strcmp (rdptr->CgiRequestMethodPtr, "POST"))
   {
      CGIVARNULL (OperaChildPtr, "FORM_OPERA_CHILD");
      if (!OperaChildPtr || *OperaChildPtr == '0') OperaChildPtr = "1";
   }
   else
      OperaChildPtr = "0";

   fprintf (stdout,
"<FORM NAME=\"soymail_form\" ID=\"soymail_form\" ACTION=\"%s\" \
METHOD=\"POST\" ENCTYPE=\"multipart/form-data\" \
ONSUBMIT=\"clientTime()\">\n\
<INPUT TYPE=\"hidden\" NAME=\"soymail_history\" \
ID=\"soymail_history\" VALUE=\"%d\">\n\
<INPUT TYPE=\"hidden\" NAME=\"newmail_refresh\" \
ID=\"newmail_refresh\" VALUE=\"\">\n\
<INPUT TYPE=\"hidden\" NAME=\"client_time\" ID=\"client_time\" VALUE=\"\">\n\
<INPUT TYPE=\"hidden\" NAME=\"opera_child\" ID=\"opera_child\" VALUE=\"%s\">\n\
<SCRIPT LANGUAGE=\"JavaScript\">\
document.write(\'<INPUT TYPE=\"hidden\" NAME=\"client_javascript\" VALUE=\"\">\');\
</SCRIPT>\n\
\n\
<TABLE WIDTH=\"100%%\" CELLPADDING=\"0\" CELLSPACING=\"5\" ALIGN=\"center\">\n\
<TR><TD>\n\n",
            rdptr->FormAction,
            SoyMailHistory,
            OperaChildPtr);
}

/*****************************************************************************/
/*
*/

void MainMenuPageEnd (REQUEST_DATA *rdptr)

{
   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuPageEnd()");

   fprintf (stdout, "</TD></TR>\n</TABLE>\n\n");

   RequestPrivateStateSave (rdptr);

   fprintf (stdout,
"</FORM>\n\
<SPAN ID=\"you_got_mail\"></SPAN>\n\
<!-- %s -->\n\
</BODY>\n\
</HTML>\n",
           StatTimer(TRUE));
}

/*****************************************************************************/
/*
*/

void MainMenuPlainPageBegin (REQUEST_DATA *rdptr)

{
   char  *UserNamePtr;
   USER_OPTIONS  *uoptr;
   VMS_MAIL_MSG  *vmptr;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuPlainPageBegin()");

   uoptr = &rdptr->UserOptions;
   muptr = &VmsMailUser;
   vmptr = &muptr->VmsMailMsg;

   if (rdptr->CharSetPtr) CgiLibResponseSetCharset (rdptr->CharSetPtr);

   CgiLibResponseHeader (200, "text/html", "%s", rdptr->LoginSetCookiePtr);

   fprintf (stdout,
"<HTML>\n\
<HEAD>\n\
<META NAME=\"generator\" CONTENT=\"%s\">\n\
<META NAME=\"environment\" CONTENT=\"%s\">\n\
<META NAME=\"copyright\" CONTENT=\"%s - GPL licensed\">\n\
<META NAME=\"language\" CONTENT=\"%s %s\">\n\
\
<STYLE TYPE=\"text/css\">\n\
body { font-family:%s; font-size:%dpt; }\n\
* { font-family:inherit; font-size:inherit; }\n\
.p0 { margin-top:0; margin-bottom:0; }\n\
.p10 { margin-top:10px; margin-bottom:0; }\n\
</STYLE>\n\
\
<TITLE>%s</TITLE>\n\
</HEAD>\n\n\
<BODY>\n\
\n\
<TABLE WIDTH=\"100%%\" CELLPADDING=\"5\" ALIGN=\"center\">\n\
<TR><TD>\n\n",
            SoftwareEnv,
            CgiEnvironmentPtr,
            SoftwareCopy,
            LangFor("lang_name"),
            LangFor("lang_author"),
            uoptr->FontFamily,
            uoptr->FontSize,
            SoyMailPageTitle(rdptr));
}

/*****************************************************************************/
/*
*/

void MainMenuPlainPageEnd (REQUEST_DATA *rdptr)

{
   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuPlainPageEnd()");

   fprintf (stdout,
"</TD></TR>\n\
</TABLE>\n\n\
</BODY>\n\
</HTML>\n");
}

/*****************************************************************************/
/*
Display the top-level main menu.
*/

void MainMenuBar (REQUEST_DATA *rdptr)

{
   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuBar()");

   MainMenuBarBegin (rdptr);
   if (rdptr->PrevPageIdent == PAGE_HELP)
      FolderOpenHelp (rdptr);
   else
   {
      fprintf (stdout, "<TD NOWRAP ALIGN=\"left\" VALIGN=\"middle\">\n\n");
      FolderListInit (rdptr);
      FolderOpenSelect (rdptr);
      fprintf (stdout, "</TD>\n");
   }
   MainMenuBarButtons (rdptr);
   MainMenuBarEnd (rdptr);
}

/*****************************************************************************/
/*
*/

void MainMenuBarBegin (REQUEST_DATA *rdptr)

{
   char  *cptr, *sptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuBarBegin()");

   fprintf (stdout,
"<!-- main menu begin -->\n\
<TABLE WIDTH=\"100%%\" CELLPADDING=\"1\" CELLSPACING=\"0\" \
ALIGN=\"center\" CLASS=\"css_panel_1\">\n\
<TR><TD>\n\
<TABLE CLASS=\"css_panel_2\" WIDTH=\"100%%\" ALIGN=\"center\" \
CELLPADDING=\"4\" CELLSPACING=\"2\">\n");

   if (SoyMailConfig.PageTitle1 &&
       SoyMailConfig.PageTitle2)
   {
      if (strchr (SoyMailConfig.PageTitle1, '<'))
         cptr = " STYLE=\"padding-left:0.5em;\"";
      else
         cptr =
" STYLE=\"font-size:120%;font-weight:bold;padding-left:0.5em;\"";

      if (strchr (SoyMailConfig.PageTitle2, '<'))
         sptr = " STYLE=\"padding-right:0.5em;\"";
      else
         sptr =
" STYLE=\"font-size:120%;font-weight:bold;padding-right:0.5em;\"";

      fprintf (stdout,
"<TR><TD ALIGN=\"left\" COLSPAN=\"2\" WIDTH=\"100%%\"%s>%s</TD>\
<TD ALIGN=\"right\" WIDTH=\"100%%\"%s>%s</TD></TR>\n",
               cptr, SoyMailConfig.PageTitle1,
               sptr, SoyMailConfig.PageTitle2);
   }
   else
   if (SoyMailConfig.PageTitle1)
   {
      if (strchr (SoyMailConfig.PageTitle1, '<'))
         cptr = "";
      else
         cptr =
" STYLE=\"font-size:120%;font-weight:bold;text-align:center;\"";

      fprintf (stdout, "<TR><TD COLSPAN=\"3\"%s>%s</TD></TR>\n",
               cptr, SoyMailConfig.PageTitle1);
   }

   fprintf (stdout,
"<TR>\n\
<TD NOWRAP ALIGN=\"center\" VALIGN=\"middle\" WIDTH=\"1%%\">&nbsp;&nbsp;");

   if (!strcmp (SOY_MAIL, "soyMAIL"))
      fprintf (stdout,
"<SPAN STYLE=\"font-weight:bold;font-size:120%%;letter-spacing:1px;\">so</SPAN>\
<SPAN STYLE=\"font-weight:bold;font-size:120%%;letter-spacing:2px;\">yMAIL</SPAN>");
   else
      fprintf (stdout,
"<SPAN STYLE=\"font-weight:bold;font-size:120%%;letter-spacing:2px;\">%s</SPAN>\n",
               SOY_MAIL);

   fprintf (stdout, "&nbsp;&nbsp;</TD>\n\n");
}

/*****************************************************************************/
/*
*/

void MainMenuBarEnd (REQUEST_DATA *rdptr)

{
   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuBarEnd()");

   fprintf (stdout,
"</TR>\n\
</TABLE>\n\
</TD></TR>\n\
</TABLE>\n\
<!-- main menu end -->\n\n");

}

/*****************************************************************************/
/*
*/

void MainMenuBarButtons (REQUEST_DATA *rdptr)

{
   BOOL  LogoutButton;
   char  *cptr, *sptr,
         *LogoutConfirm;
   VMS_MAIL_USER  *muptr;

   /*********/
   /* begin */
   /*********/

   if (WatchEnabled) WatchThis ("MainMenuBarButtons()");

   muptr = &VmsMailUser;

   fprintf (stdout,
"<!-- main menu buttons begin -->\n\
<TD NOWRAP ALIGN=\"right\" VALIGN=\"middle\">\n\
<INPUT CLASS=\"css_button_panel_2\"%s TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\">&nbsp;\n\
<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\">&nbsp;\n\
<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\">&nbsp;\n\
<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\">",
           rdptr->ThisPageIdent == PAGE_HELP || SoyMailConfig.SearchEnabled ?
              "" : " STYLE=\"font-style:italic;\"",
           LangFor("search"),
           LangFor("options"),
           LangFor("contacts"),
           LangFor("compose"));

   if (rdptr->ThisPageIdent != PAGE_HELP)
      if (rdptr->ClientHasJavaScript)
         fprintf (stdout,
"<SCRIPT LANGUAGE=\"JavaScript\">\
document.write(\'<INPUT CLASS=\"css_button_panel_2\" \
STYLE=\"border-left-width:0px;\" \
TYPE=\"submit\" NAME=\"main_btn\" VALUE=\"^\" \
ONCLICK=\"targetBlank();\">\');\
</SCRIPT>");

   fprintf (stdout, "&nbsp;\n");

   if (rdptr->ThisPageIdent == PAGE_HELP)
   {
      fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\">&nbsp;\n",
               LangFor("logout"));
   }
   else
   {
      LogoutButton = FALSE;
      if (SoyMailConfig.LoginSecret)
      {
         /* using soyMAIL's own internal authentication */
         LogoutButton = TRUE;
         LogoutConfirm = LANG_FOR_CONFIRM("login_confirm_logout");
      }
      else
      if (CgiLibEnvironmentIsOsu())
      {
         /* OSU does not appear to provide AUTH_TYPE */
         LogoutButton = TRUE;
         LogoutConfirm = LANG_FOR_CONFIRM("logout_confirm");
      }
      else
      {
         /* if it's BASIC or DIGEST authentication */
         CGIVARNULL (cptr, "AUTH_TYPE");
         if (!cptr) cptr = "";
         if (strsame(cptr,"basic",-1) || strsame(cptr,"digest",-1))
         {
            LogoutButton = TRUE;
            LogoutConfirm = LANG_FOR_CONFIRM("logout_confirm");
         }
      }

      if (LogoutButton)
      {
         if (SoyMailConfig.LogoutDisabled)
            fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_2\" STYLE=\"font-style:italic;\" \
TYPE=\"submit\" ID=\"main_btn_logout\" NAME=\"main_btn\" VALUE=\"%s\" \
ONCLICK=\"return false\">&nbsp;\n",
                     LangFor("logout"));
          else
            fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_2\" \
TYPE=\"submit\" NAME=\"main_btn\" VALUE=\"%s\" \
ONCLICK=\"return confirm(\'%s\')\">&nbsp;\n",
                     LangFor("logout"),
                     LogoutConfirm);
      }
   }

   if (rdptr->ClientHasJavaScript)
      fprintf (stdout,
"<SCRIPT LANGUAGE=\"JavaScript\">\
if (childWindow()) \
document.write(\'<INPUT CLASS=\"css_button_panel_2\" \
TYPE=\"submit\" NAME=\"main_btn\" VALUE=\"%s\" \
ONCLICK=\"return confirmClose(%s)\">&nbsp;\\n\');\
</SCRIPT>",
              LangFor("close"),
              rdptr->ThisPageIdent == PAGE_COMPOSE &&
                 !rdptr->ComposeData.AlreadySent ? "false" : "true");

   fprintf (stdout,
"<INPUT CLASS=\"css_button_panel_2\" TYPE=\"submit\" \
NAME=\"main_btn\" VALUE=\"%s\" \
ONCLICK=\"document.soymail_form.target=\'soymail_help\'\">&nbsp;\n\
</TD>\n\
<!-- main menu buttons end -->\n\
\n",
           LangFor("help"));

}

/*****************************************************************************/

